using ComPDFKit.PDFDocument;
using Compdfkit_Tools.Data;
using Compdfkit_Tools.Helper;
using Compdfkit_Tools.PDFControl;
using ComPDFKitViewer;
using ComPDFKitViewer.AnnotEvent;
using ComPDFKitViewer.PdfViewer;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Runtime.CompilerServices;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media.Imaging;
namespace AnnotationViewControl
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow: Window, INotifyPropertyChanged
    {
        private PDFViewControl pdfViewer;
        private PDFViewControl passwordViewer;
        private AnnotationControl annotationControl = new AnnotationControl();
        private CPDFBOTABarControl botaBarControl = new CPDFBOTABarControl(BOTATools.Thumbnail | BOTATools.Outline | BOTATools.Bookmark | BOTATools.Search | BOTATools.Annotation);
        
        private bool _canSave = false;
        public bool CanSave
        {

            get => _canSave;
            set
            {
                _canSave = value;
                OnPropertyChanged();
            }
        }
        
        public MainWindow()
        {
            InitializeComponent();
            DataContext = this;
        }

        #region Load document

        private void LoadDefaultDocument()
        {
            string defaultFilePath = "PDF32000_2008.pdf";
            pdfViewer.PDFView.InitDocument(defaultFilePath);
            LoadDocument();
        }

        private void LoadDocument()
        {
            pdfViewer.PDFView.Load();
            pdfViewer.PDFView.SetShowLink(true);
            
            pdfViewer.PDFView.InfoChanged -= PdfViewer_InfoChanged;
            pdfViewer.PDFView.InfoChanged += PdfViewer_InfoChanged;
            PDFGrid.Child = annotationControl;
            
            annotationControl.PdfViewControl = pdfViewer;
            annotationControl.InitWithPDFViewer(pdfViewer.PDFView);
            InitialPDFViewControl(annotationControl.PdfViewControl);

            annotationControl.OnCanSaveChanged -= AnnotationControl_OnCanSaveChanged;
            annotationControl.OnCanSaveChanged += AnnotationControl_OnCanSaveChanged;
            annotationControl.OnAnnotEditHandler -= PdfAnnotationControl_RefreshAnnotList;
            annotationControl.OnAnnotEditHandler += PdfAnnotationControl_RefreshAnnotList;
            
            annotationControl.PdfViewControl.PDFView.SetFormFieldHighlight(true);
            PasswordUI.Closed -= PasswordUI_Closed;
            PasswordUI.Canceled -= PasswordUI_Canceled;
            PasswordUI.Confirmed -= PasswordUI_Confirmed;
            PasswordUI.Closed += PasswordUI_Closed;
            PasswordUI.Canceled += PasswordUI_Canceled;
            PasswordUI.Confirmed += PasswordUI_Confirmed;

            annotationControl.PdfViewControl.PDFView.ChangeFitMode(FitMode.FitWidth);
            CPDFSaclingControl.InitWithPDFViewer(annotationControl.PdfViewControl.PDFView);
            CPDFSaclingControl.SetZoomTextBoxText(string.Format("{0}", (int)(annotationControl.PdfViewControl.PDFView.ZoomFactor * 100)));

            ViewSettingBtn.IsChecked = false;
            botaBarControl.InitWithPDFViewer(annotationControl.PdfViewControl.PDFView);
            botaBarControl.SelectBotaTool(BOTATools.Thumbnail);
            annotationControl.SetBOTAContainer(botaBarControl);
            annotationControl.InitialPDFViewControl(annotationControl.PdfViewControl);
        }

        private void PdfAnnotationControl_RefreshAnnotList(object sender, EventArgs e)
        {
            botaBarControl.LoadAnnotationList();
        }

        private void AnnotationControl_OnCanSaveChanged(object sender, bool e)
        {
           this.CanSave = e;
        }

        #endregion
        
        #region Password

        private void PasswordUI_Confirmed(object sender, string e)
        {
            if (passwordViewer != null && passwordViewer.PDFView != null && passwordViewer.PDFView.Document != null)
            {
                passwordViewer.PDFView.Document.UnlockWithPassword(e);
                if (passwordViewer.PDFView.Document.IsLocked == false)
                {
                    PasswordUI.SetShowError("", Visibility.Collapsed);
                    PasswordUI.ClearPassword();
                    PasswordUI.Visibility = Visibility.Collapsed;
                    PopupBorder.Visibility = Visibility.Collapsed;
                    pdfViewer = passwordViewer;
                    LoadDocument();
                }
                else
                {
                    PasswordUI.SetShowError("Wrong Password", Visibility.Visible);
                }
            }
        }

        private void PasswordUI_Canceled(object sender, EventArgs e)
        {
            PopupBorder.Visibility = Visibility.Collapsed;
            PasswordUI.Visibility = Visibility.Collapsed;
        }

        private void PasswordUI_Closed(object sender, EventArgs e)
        {
            PopupBorder.Visibility = Visibility.Collapsed;
            PasswordUI.Visibility = Visibility.Collapsed;
        }

        #endregion
        
        #region Load Unload custom control

        
        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            pdfViewer = new PDFViewControl();
            LoadDefaultDocument();
        }
        #endregion
        
        #region Annotation

        public void InitialPDFViewControl(PDFViewControl newPDFViewer)
        {
            annotationControl.ClearAllToolState();
            annotationControl.ExpandRightPropertyPanel(null, Visibility.Collapsed);
        }
          
        #endregion
        
        #region Event handle

        private void PdfViewer_InfoChanged(object sender, KeyValuePair<string, object> e)
        { 
            if (e.Key == "Zoom")
            {
                CPDFSaclingControl.SetZoomTextBoxText(string.Format("{0}", (int)((double)e.Value * 100)));
            }
        }
        
        
        
        #endregion
        
        private void SaveFileBtn_Click(object sender, RoutedEventArgs e)
        {
            SaveFile();
            pdfViewer.PDFView.UndoManager.CanSave = false;
        }
        
        private void OpenFile()
        {
            string filePath = CommonHelper.GetFilePathOrEmpty();
            if (!string.IsNullOrEmpty(filePath) && annotationControl.PdfViewControl != null)
            {
                if (pdfViewer.PDFView != null && pdfViewer.PDFView.Document != null)
                {
                    string oldFilePath = pdfViewer.PDFView.Document.FilePath;
                    if (oldFilePath.ToLower() == filePath.ToLower())
                    {
                        return;
                    }
                }

                passwordViewer = new PDFViewControl();
                passwordViewer.PDFView.InitDocument(filePath);
                if (passwordViewer.PDFView.Document == null)
                {
                    MessageBox.Show("Open File Failed");
                    return;
                }

                if (passwordViewer.PDFView.Document.IsLocked)
                {
                    PasswordUI.SetShowText(System.IO.Path.GetFileName(filePath) + " password encrypted.");
                    PasswordUI.ClearPassword();
                    PopupBorder.Visibility = Visibility.Visible;
                    PasswordUI.Visibility = Visibility.Visible;
                }
                else
                {
                    pdfViewer = passwordViewer;
                    LoadDocument();
                }
            }
        }

        private void OpenFile_Click(object sender, RoutedEventArgs e)
        {
            OpenFile();
        }

        private void LeftToolPanelButton_Click(object sender, RoutedEventArgs e)
        {
            ToggleButton expandBtn = sender as ToggleButton;
            if (expandBtn != null)
            {
                bool isExpand = expandBtn.IsChecked == true;
                annotationControl.ExpandLeftPanel(isExpand);
            }
        }

        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var item = (sender as ComboBox).SelectedItem as ComboBoxItem;
            if ((string)item.Content == "Viewer")
            {
                annotationControl.ClearAllToolState();
                annotationControl.SetToolBarContainerVisibility(Visibility.Collapsed);
                annotationControl.ExpandRightPropertyPanel(null, Visibility.Collapsed);
                annotationControl.PDFAnnotationControl.AnnotationCancel();
                RightPanelButton.IsChecked = false;
                if (annotationControl.PdfViewControl != null && annotationControl.PdfViewControl.PDFView != null)
                {
                    annotationControl.PdfViewControl.PDFView.SetMouseMode(MouseModes.Viewer);
                }
            }
            else if ((string)item.Content == "Annotation")
            {
                annotationControl.SetToolBarContainerVisibility(Visibility.Visible);
                if (annotationControl.PdfViewControl != null && annotationControl.PdfViewControl.PDFView != null)
                {
                    annotationControl.PdfViewControl.PDFView.SetMouseMode(MouseModes.PanTool);
                }
            }
        }

        private void PageInfoBtn_Click(object sender, RoutedEventArgs e)
        {
            PasswordUI.Visibility = Visibility.Collapsed;
            FileInfoUI.Visibility = Visibility.Visible;
            FileInfoControl.InitWithPDFViewer(pdfViewer.PDFView);
            PopupBorder.Visibility = Visibility.Visible;
        }

        private void ViewSettingBtn_Click(object sender, RoutedEventArgs e)
        {
            ShowViewSettings();
        }

        private void ShowViewSettings()
        {
            if (ViewSettingBtn != null)
            {
                if (ViewSettingBtn.IsChecked == true)
                {
                    CPDFDisplaySettingsControl displayPanel = new CPDFDisplaySettingsControl();
                    displayPanel.InitWithPDFViewer(annotationControl.PdfViewControl.PDFView);
                    annotationControl.SetViewSettings(Visibility.Visible, displayPanel); 
                    if ((bool)RightPanelButton.IsChecked)
                    {
                        RightPanelButton.IsChecked = false;
                    }
                }
                else
                {
                    annotationControl.SetViewSettings(Visibility.Collapsed); 
                }
            }
        }
        private void RightPanelButton_Click(object sender, RoutedEventArgs e)
        {
            ControlRightPanel();
        }

        private void ExpandSearchBtn_Click(object sender, RoutedEventArgs e)
        {
            annotationControl.ExpandLeftPanel(true);
            botaBarControl.SelectBotaTool(BOTATools.Search);
        }
        
        private void ControlRightPanel()
        { 
            if (RightPanelButton != null)
            {
                if (RightPanelButton.IsChecked == true)
                {
                    if (annotationControl.PDFAnnotationControl != null)
                    {
                        annotationControl.ExpandRightPropertyPanel(annotationControl.PDFAnnotationControl, Visibility.Visible);
                        if ((bool)ViewSettingBtn.IsChecked)
                        {
                            ViewSettingBtn.IsChecked = false;
                        }
                    }
                }
                else
                {
                    annotationControl.ExpandRightPropertyPanel(null, Visibility.Collapsed);
                }
            }
        }

        #region Save file
        /// <summary>
        /// Save the file to another PDF file.
        /// </summary>
        public void SaveAsFile()
        {
            {
                if (pdfViewer != null && pdfViewer.PDFView != null && pdfViewer.PDFView.Document != null)
                {
                    CPDFDocument pdfDoc = pdfViewer.PDFView.Document;
                    SaveFileDialog saveDialog = new SaveFileDialog();
                    saveDialog.Filter = "(*.pdf)|*.pdf";
                    saveDialog.DefaultExt = ".pdf";
                    saveDialog.OverwritePrompt = true;

                    if (saveDialog.ShowDialog() == true)
                    {
                        pdfDoc.WriteToFilePath(saveDialog.FileName);
                    }
                }
            }
        }

        /// <summary>
        /// Save the file in the current path.
        /// </summary>
        private void SaveFile()
        {
            if (pdfViewer != null && pdfViewer.PDFView != null && pdfViewer.PDFView.Document != null)
            {
                try
                {
                    CPDFDocument pdfDoc = pdfViewer.PDFView.Document;
                    if (pdfDoc.WriteToLoadedPath())
                    {
                        return;
                    }

                    SaveFileDialog saveDialog = new SaveFileDialog();
                    saveDialog.Filter = "(*.pdf)|*.pdf";
                    saveDialog.DefaultExt = ".pdf";
                    saveDialog.OverwritePrompt = true;

                    if (saveDialog.ShowDialog() == true)
                    {
                        pdfDoc.WriteToFilePath(saveDialog.FileName);
                    }
                }
                catch (Exception ex)
                {

                }
            }
        }
        #endregion

        private void FileInfoCloseBtn_Click(object sender, RoutedEventArgs e)
        {
            PopupBorder.Visibility = Visibility.Collapsed;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}